//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#include <signal.h>
#include <string>
#include <list>
#include <cstdio>
#include <unistd.h>
#include <libvtql/error_base.h>
#include "exec.h"
#include "../config.h"

const char cpQuerySocket_dfl[]=VTQL_QUERY_SOCKET_DEFAULT;
const char cpEventSocket_dfl[]=VTQL_EVENT_SOCKET_DEFAULT;


const char cpUsage[]=
"Usage: VTQLclient <-h|-v>|<[-q] <[-qs socket] [-es socket] [-e]>|[-s socket]\n"
"  [command_1 [command_2 [ ... ]]] -n>\n"
"\n"
" Examples:\n"
"  VTQLclient -v\n"
"  VTQLclient \"SHOW STATION\"\n"
"  VTQLclient -es $(HOME)/.myeventsocket -e \"PAGE ..100 LIST\"\n"
"  VTQLclient -s $(HOME)/.myeventsocket -n \"PAGE ..100 LIST\"\n"
"  echo \"SHOW FREQ\" | VTQLclient \n"
"\n"
" Options:\n"
"  -h,--help                how this text and exit\n"
"  -v,--version             show version info and exit\n" 
"  -q,--quiet               do not show greeting, this is default, if no tty\n"
"                           or executing commands form command line\n"
"  -qs,--querysocket <name> socket to VTQLserver in query mode [%s]\n"
"                           (you may also use $VTQL_SOCKET)\n"
"  -qs,--eventsocket <name> socket to VTQLserver in event mode [%s]\n"
"                           (you may also use $VTQL_SOCKET_EVENT)\n"
"  -e,--event               Use the event socket. This includes -n.\n"
"  -s,--socket <name>       socket to VTQLserver in any mode\n"
"  command_n                Commands to execute. A \"QUIT\" command will be\n"
"                           added as last command. If you do not add commands\n"
"                           in command line, the commands will be read from\n"
"                           stdin. A EOF at stdin will send a QUIT to the\n"
"                           server\n"
"  -n,--noquit              Do not send \"QUIT\" command automatically\n";

const char cpVersion[]=
"VTQLclient " VTQLCLIENT_VERSION "\n"
"Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>";

bool DblT(const std::string& sC,const std::string& sA,
 const std::string& sA2=""){
 return (sC==sA || sC==((sA2=="")?(sA.substr(0,1)+sA[2]):sA2));
}

int main(int iArgs,char** cppArgs){try{
 signal(SIGPIPE,SIG_IGN);
 signal(SIGHUP,SIG_IGN);
 std::list<std::string> clArgs;
 for(int i=1;i<iArgs;i++)clArgs.push_back(cppArgs[i]);

 std::string sQSock=(
  getenv("VTQL_SOCKET")?getenv("VTQL_SOCKET"):cpQuerySocket_dfl);
 std::string sEvSock=(
  getenv("VTQL_SOCKET_EVENT")?getenv("VTQL_SOCKET_EVENT"):cpEventSocket_dfl);
 bool bQuiet=false;
 bool bEvent=false;
 std::string sSock="";
 bool bNoQuit=false;

 try{
 for(std::list<std::string>::iterator i=clArgs.begin();i!=clArgs.end();){
  if(DblT(*i,"--help")){
   std::printf(cpUsage,sQSock.c_str(),sEvSock.c_str());
   return 0;
  }else if(DblT(*i,"--version")){
   std::puts(cpVersion);
   return 0;
  }else if(DblT(*i,"--quiet")){
   bQuiet=true;
   i=clArgs.erase(i);
  }else if(DblT(*i,"--querysocket","-qs")){
   if(clArgs.size()<2)throw std::string("usage");
   i=clArgs.erase(i);
   sQSock=*i;
   i=clArgs.erase(i);
  }else if(DblT(*i,"--eventsocket","-es")){
   if(clArgs.size()<2)throw std::string("usage");
   i=clArgs.erase(i);
   sEvSock=*i;
   i=clArgs.erase(i);
  }else if(DblT(*i,"--event")){
   bEvent=true;
   bNoQuit=true;
   i=clArgs.erase(i);
  }else if(DblT(*i,"--socket")){
   if(clArgs.size()<2)throw std::string("usage");
   i=clArgs.erase(i);
   sSock=*i;
   i=clArgs.erase(i);
  }else if(DblT(*i,"--noquit")){
   bNoQuit=true;
   i=clArgs.erase(i);
  }else ++i;
 }
 if(sSock=="")sSock=(bEvent?sEvSock:sQSock);
 bool bTTY=(ttyname(0)!=NULL);
 if(!bTTY)bQuiet=true;
 if(!clArgs.empty()){
  bQuiet=true;
  if(!bNoQuit)clArgs.push_back("QUIT");
 }

 thread::Semaphore sem;
 sock::Socket_basic socket;socket.Connect(sSock);
 
 Output out(sem,socket,bQuiet);
 Input* in=(clArgs.empty()?new Input(sem,bNoQuit,bTTY):NULL);
 size_t iPos=0;


 out.Start<Output>();
 if(in)in->Start<Input>();
 sock::PollClass p;p.AddSocket(socket,POLLOUT);

 while(true){
  if(in||clArgs.empty())sem.Wait();
  char c=out;
  if(c)std::putchar(c); 
  else if(!socket.IsOpen())break;
  if(in)c=*in;
  else if(!clArgs.empty()){
   if(iPos>clArgs.front().size()){
    c='\n';
    clArgs.pop_front();
    iPos=0;
   }else c=clArgs.front()[iPos++];
  }else c='\0';
  if(c){
   p.Run(-1);
   write(socket,&c,1);
  }
 }
 out.Stop();
// if(in)in->Stop();

 }catch(std::string s){
  if(s!="usage")throw Error::Runtime(METH_NAME,s);
  std::fprintf(stderr,cpUsage,sQSock.c_str(),sEvSock.c_str());
  return 1;
 }
 }catch(Error::base e){
  std::printf("\nfatal: %s\n",std::string(e).c_str());
  return 1;
 }
 return 0;
}
