//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#include "connection.event.h"

#include <libvtql/vtql_utils.h>
#include "glob_defs.h"
#include "server.h"
#include "db.h"

using std::string;

Connection_event::Connection_event(int iS,VTQL::UserMode um,Server& s):
Connection_base(iS,um),server(s),iCallBID(s.vt_dbp->cb.Add(this)),
 Inp(Con,glob_defs::iMaxInpBytes),Outp(Con,glob_defs::iMaxOutpLines),
 el(VTQL::SM_LIST){

 Outp.Add(glob_defs::cpVersion);
 Outp.Add(glob_defs::cpGreeting);

 SetMatch(".none");
}

Connection_event::~Connection_event(){
 server.vt_dbp->cb.Del(iCallBID);
}

bool Connection_event::Add2Poll(SockWait& p)const{
 short s=POLLIN;
 if(!Outp.Empty())s|=POLLOUT;
 p.AddSocket(Con,s);
 if(!Outp.Full() && Inp.Lines())return false;
 return true;
}

bool Connection_event::Run(bool){
 bool bW=true,bR=true;
 while(bR || bW){
  if(!Inp.Free())bR=false;
  if(Outp.Empty())bW=false;
  if(!Poll(bR,bW))return false;

  if(bW)Outp.Write();
  bW=false;
  if(bR)Inp.Read();
  bR=false;
 }
 try{
  std::pair<string,bool> s=Inp.GetLine();
  if(!s.second)return true;
  VTQL::Parse vtql;
  vtql.NewString(s.first);
  int iQ=vtql.GetCom();
  if(iQ<0)return true;
  switch(iQ){
   #define EXEC_COM(n,u,c) case n:{c} break;
   #define INC_EVENT
   #include "vtql.syntax.inc"
   #undef INC_EVENT
   #undef EXEC_COM
   default:throw Error::Syntax(METH_NAME,"bad command");
  }
 }catch(Error::Syntax){}

 return true;
}

void Connection_event::SetMatch(const string& s){
 SearchQueue::SmartRP p(new Region());
 p->NewString(s,"|\\");
 regMatch=p;
 sMatch=s;
}

void Connection_event::operator ()(const Arg& p){
 if(Outp.Full())return;
 if(!regMatch->MatchPage(p))return;
 if(!regMatch->MatchStation(p.iStID))return;

 Outp<<("PAGE "+VTQL::EscapeString(sMatch)+" "+VTQL::cppShowModes[el]);
 Outp.GetPage(p.PD->pmd,el);
 Outp<<p;
 server.wc.cbSocket(Con);
}

