//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#include "server.h"

#include "connection.event.h"
#include "connection.exec.h"
#include "db.h"
#include "vtrun.h"
#include "error_class.h"
#include "glob_defs.h"

void SocketInfo::Handle(int iLockSock){
 ConMap::iterator j=lcpCon.begin();
 while(j!=lcpCon.end()){
  if(!j->second->Run(iLockSock>=0 && iLockSock!=j->second->GetSock())){
   ConMap::iterator js=j++;
   lcpCon.erase(js);
  }else ++j;
 }

 if(sbSock.IsOpen()){
  sock::PollClass pc;
  pc.AddSocket(sbSock,POLLIN);
  if(pc.Run(0)>0){
   if(pc.Test(0,POLLIN)){
    Connection_base* cbp=NULL;
    switch(um){
     case VTQL::UM_NOBODY:
     case VTQL::UM_CONFIG:
     case VTQL::UM_QUERY :cbp=new Connection_exec (sbSock,um,server);break;
     case VTQL::UM_EVENT :cbp=new Connection_event(sbSock,um,server);break;
    }
    lcpCon.Add(cbp);
   }
   else if(pc.Test(0,POLLNVAL))Close();
  }
 }
}

void Server::Add(size_t id,const std::string& sFileN){
 SocketInfo* sip=new SocketInfo(sFileN,VTQL::UM_NOBODY,*this);
 if(smSockets.AddIf(id,sip)!=sip)
  throw Error::Exec(METH_NAME,"socket ID already exists");
}

void Server::Del(size_t id){
 if((*this)[id].IsOpen())
  throw Error::Exec(METH_NAME,"socket still open");
 smSockets.erase(id);
}

SocketInfo& Server::operator [](size_t id){
 SocketInfo* sip=smSockets[id];
 if(!sip)throw Error::Exec(METH_NAME,"socket ID does not exist");
 return *sip;
}

void Server::CleanUp(){
 for(SocketMap::iterator i=smSockets.begin();i!=smSockets.end();++i)
  if(!i->second->sbSock.IsOpen())i->second->lcpCon.clear();
 pAll.Clear();
}

bool Server::FillUp(){
 for(SocketMap::iterator i=smSockets.begin();i!=smSockets.end();++i)
  if(i->second->IsOpen()){
   for(SocketInfo::ConMap::iterator j=i->second->lcpCon.begin();
       j!=i->second->lcpCon.end();++j)
    if(!j->second->Add2Poll(pAll))return false;
   pAll.AddSocket(i->second->sbSock,POLLIN);
  }
 return true;
}

void Server::Handle(){
 vt_runp=new VT_Run;
 vt_dbp=new Database;
 vt_dbp->sqSearch.cbMatch.Add(&wc.cbSearch); 
 pAll.cbPoll.Add(&wc.cbSocket);
 vt_runp->ptGetData.cbPage.Add(&wc.cbPage);
 
 pAll.SetWait(glob_defs::iMainPollWait);
 CleanUp();
 FillUp();

 pAll.ManageThread(true);
 vt_dbp->sqSearch.ManageThread(true);

 while(true){//well this is the main loop :)
  if(smSockets.empty())throw Error::Program(METH_NAME,"no socket open");

  vt_runp->Autozap(); 
  const vt_tv::PageTyp* vtp;
  if((vtp=wc.cbPage.fifo.Get_Start(false))){
   vt_dbp->Insert(*vtp);
   cbEvent.Run(*vtp);
   wc.cbPage.fifo.Get_End(true);
  }

  pAll.Lock();
  CleanUp();
  bool b=!FillUp();
  pAll.Restart();
  pAll.UnLock();

  if(!b)b=wc.Wait();

  if(b)for(SocketMap::iterator i=smSockets.begin();i!=smSockets.end();++i)
   i->second->Handle(iLockSock);
 };
 vt_dbp->sqSearch.ManageThread(false);
 delete vt_runp;vt_runp=NULL;
 delete vt_dbp;vt_dbp=NULL;
}
