//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#include "vtrun.h"

#include <cstdio>
#include <libvtql/sock.h>
#include "vtpage.h"
#include "error_class.h"
#include <libvtql/vtql_utils.h>

using namespace vt_tv;
using std::string;

bool VT_Run::SetFreq_U(int iF){
 if(iF>0 && iF==iActF)return true;
 if(iVideodev>=0){
  if(!VT_TV::SetFreq_U(iF))return false;
  iLastF=iActF; 
 }else iLastF=((iF>0)?iF:-1);
 tLastZap=time(0);tStop=tLastZap;
 return true;
}

void* VT_Run::PageThread::Run(){
 PageTyp vtp;
 while(!ExitRun()){
   if(vt_run.GetVT(vtp,80))cbPage.Run(vtp);
 }
 return NULL;
}

VT_Run::VT_Run():VT_TV(),ptGetData(*this),iLastF(-1),tLastZap(0),tStop(0){}
VT_Run::~VT_Run(){if(Vbi())ptGetData.Stop();}

void VT_Run::Autozap(){
 if(!Video() || !Vbi() || !azmFreq.size())return;

 AutozapMap::const_iterator sti=azmFreq.find(iLastF);
 if(sti==azmFreq.end() || sti->second > time(0)-tLastZap)return;

 ++sti;if(sti==azmFreq.end())sti=azmFreq.begin(); //increase

 Lock();
 //unsuccessfull exec does not matter
 SetFreq_U(sti->first); 
 Unlock();
}


void VT_Run::SetFreq(int iF){
 if(iF==0)throw Error::Exec(METH_NAME,"invalid frequency");
 Lock();
 if(!SetFreq_U(iF)){
  Unlock();
  throw Error::Exec(METH_NAME,"cannot set frequency");
 }
 Unlock();
}

void VT_Run::OpenVideo(){
 if(Video())return;
 if(!VT_TV::OpenVideo())
  throw Error::Exec(METH_NAME,"cannot open video device");

 if(Vbi())tLastZap=time(0)-(tStop-tLastZap);

 Lock();
 if(iLastF<0){iLastF=iActF;tLastZap=time(0);tStop=tLastZap;}
 else{
  if(!VT_TV::SetFreq_U(iLastF)){
   Unlock();
   throw Error::Exec(METH_NAME,"cannot set frequency");
  }
  iLastF=iActF;
 }
 Unlock();
}

void VT_Run::CloseVideo(){
 if(!Video())return;
 iLastF=iActF;
 VT_TV::CloseVideo();
 if(Vbi())tStop=time(0);
}

void VT_Run::OpenVbi(){
 if(Vbi())return;
 if(!VT_TV::OpenVbi())throw Error::Exec(METH_NAME,"cannot open vbi device");
 if(!ptGetData.Start<VT_Run::PageThread>())
  throw Error::Runtime(METH_NAME,"cannot start thread");
 if(Video())tLastZap=time(0)-(tStop-tLastZap);
}

void VT_Run::CloseVbi(){
 if(!Vbi())return;
 ptGetData.Stop();
 VT_TV::CloseVbi();
 if(Video())tStop=time(0);
}

void VT_Run::ChangeZap(int iF,int iAZ){
 if(iF<1)throw Error::Exec(METH_NAME,"invalid frequency");
 AutozapMap::iterator sti=azmFreq.find(iF);
 if(iAZ>0){
  if(sti!=azmFreq.end())sti->second=iAZ;
  else azmFreq.insert(AutozapMap::value_type(iF,iAZ));
 }else if(sti!=azmFreq.end())azmFreq.erase(sti);
}

void VT_Run::ShowZap(int iFreq,sock::Output& Outp){
 char cp[50];
 for(AutozapMap::iterator sti=azmFreq.begin();sti!=azmFreq.end();++sti){
  snprintf(cp,50,"%i\t%i\t",sti->first,sti->second);
  Outp.Add(string(cp)+((iFreq!=0 && iFreq==sti->first)?"":"in")+"active");
 }
}

extern "C"{
#include <libvtql/extern/frequencies.h>
}

FreqTab::GetList::GetList(){
 for(iE=0;chanlists[iE].name;++iE)
 iSt=-1;
}

FreqTab::GetList::GetList(const string& s){
 for(iSt=0;chanlists[iSt].name;++iSt)
  if(s==chanlists[iSt].name)break;
 if(!chanlists[iSt].name)
  throw Error::Exec(METH_NAME,"frequency table not found");
 iE=chanlists[iSt].count;
}


