//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>
#include "command_exec.h"
#include <cstdio>
#include <stdlib.h>
#include "../config.h"

const char cpQuerySocket_dfl[]=VTQL_QUERY_SOCKET_DEFAULT;
const char cpEventSocket_dfl[]=VTQL_EVENT_SOCKET_DEFAULT;

const char cpUsage[]=
"Usage: console_vt <-h|-v>|<[-q] [-qs socket] [-es socket]>\n"
"\n"
" Examples:\n"
"  console_vt\n"
"  console_vt -q\n"
"  console_vt -es $(HOME)/.myeventsocket\n"
"\n"
" Options:\n"
"  -h,--help                show this text and exit\n"
"  -v--version              show version and exit\n"
"  -q,--quiet               do not show greeting\n"
"  -qs,--querysocket <name> socket to VTQLserver in query mode [%s]\n"
"                           (you may also use $VTQL_SOCKET)\n"
"  -es,--eventsocket <name> socket to VTQLserver in event mode [%s]\n"
"                           (you may also use $VTQL_SOCKET_EVENT)\n";

const char cpVersion[]=
"console_vt " CONSOLEVT_VERSION "\n"
"Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>";

bool DblT(const std::string& sC,const std::string& sA,const std::string& sA2="")
{return (sC==sA || sC==((sA2=="")?(sA.substr(0,1)+sA[2]):sA2));}

int main(int iArgC,char** cppArgs){try{
 std::list<std::string> clArgs;
 for(int i=1;i<iArgC;i++)clArgs.push_back(cppArgs[i]);

 std::string sQSock=(
  getenv("VTQL_SOCKET")?getenv("VTQL_SOCKET"):cpQuerySocket_dfl); 
 std::string sEvSock=(
  getenv("VTQL_SOCKET_EVENT")?getenv("VTQL_SOCKET_EVENT"):cpEventSocket_dfl); 
 bool bQuiet=false;

 try{
 for(std::list<std::string>::iterator i=clArgs.begin();i!=clArgs.end();){
  if(DblT(*i,"--help")){
   std::printf(cpUsage,sQSock.c_str(),sEvSock.c_str());
   return 0;
  }else if(DblT(*i,"--version")){
   std::puts(cpVersion);
   return 0;
  }else if(DblT(*i,"--quiet")){
   bQuiet=true;
   i=clArgs.erase(i);
  }else if(DblT(*i,"--querysocket","-qs")){
   if(clArgs.size()<2)throw std::string("usage");
   i=clArgs.erase(i);
   sQSock=*i;
   i=clArgs.erase(i);
  }else if(DblT(*i,"--eventsocket","-es")){
   if(clArgs.size()<2)throw std::string("usage");
   i=clArgs.erase(i);
   sEvSock=*i;
   i=clArgs.erase(i);
  }else throw std::string("usage");
 }
 }catch(std::string s){
  if(s!="usage")throw Error::Runtime(METH_NAME,s);
  std::fprintf(stderr,cpUsage,sQSock.c_str(),sEvSock.c_str());
  return 1;
 }
 Run(sQSock,sEvSock,bQuiet);
 }catch(Error::base e){
  std::printf("\nfatal: %s\n",std::string(e).c_str());
 }
 return 0;
}
