/*! \file smartptr.h
    \author Matthias Kleinmann <Matthias dot Kleinmann at gmx dot de>
    \brief  Smart pointer objects.
*/
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef CIDM_H
#define CIDM_H

#include <map>

template<class T,typename IDTypeT=size_t>
class CachedExternIDMap{
 public:
  CachedExternIDMap(const CachedExternIDMap&);//no impl
  CachedExternIDMap(){pC=mD.end();}
  virtual ~CachedExternIDMap(){clear();}

  virtual T* AddIf(IDTypeT,T*);
  virtual T* operator [](IDTypeT)const;
  
  typedef std::map<IDTypeT,T*> IDMap;
  typedef typename IDMap::const_iterator const_iterator;
  typedef typename IDMap::iterator iterator;

  virtual bool empty()const{return mD.empty();}
  virtual size_t size()const{return mD.size();}
  virtual const_iterator begin()const{return mD.begin();}
  virtual const_iterator end()const{return mD.end();}
  virtual iterator begin(){return mD.begin();}
  virtual iterator end(){return mD.end();}

  virtual bool erase(IDTypeT);
  virtual void erase(iterator);
  virtual void clear();

 protected:
  IDMap mD;
  mutable const_iterator pC;
};

template <class T,typename IDTypeT=size_t>
class CachedInternIDMap:public CachedExternIDMap<T,IDTypeT>{
 public:
  CachedInternIDMap(const CachedInternIDMap&);//no impl
  explicit CachedInternIDMap(IDTypeT ID=0)
  :CachedExternIDMap<T,IDTypeT>(),iID_C(ID){};
  virtual ~CachedInternIDMap(){};

  virtual IDTypeT Add(T*);

 protected:
  IDTypeT iID_C;
};

template <typename T,typename TAdd=volatile unsigned>
class SmartPtr{
 public:
  SmartPtr(const SmartPtr<T,TAdd>&);
  SmartPtr(T* tp=NULL):ptr(tp),spRef(new TAdd(1)){};
  ~SmartPtr();

  SmartPtr<T,TAdd>& operator =(const SmartPtr<T,TAdd>&);

  operator T*()const{return ptr;}
  operator T*(){return ptr;}
  const T& operator *()const{return *ptr;}
  const T* operator ->()const{return ptr;}
  T& operator *(){return *ptr;}
  T* operator ->(){return ptr;}

  const TAdd& GetCount()const{return *spRef;}
 private:
  T* ptr;
  TAdd* spRef;
};

#include "smartptr.tcc"
#endif
