//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#include "sock.h"

#include <sys/socket.h>
#include <fcntl.h>
#include <unistd.h>
#include <cstdlib>
#include <cstdio>

using namespace sock;

void Socket_basic::PrepareOpen(const std::string& sFile,sockaddr_un& saStand){
 Close();
 bMy=true;

 if(sFile.length()>sizeof(saStand.sun_path)-1)
  throw Error::Sock(METH_NAME,"file name too long");

 iFD=socket(PF_UNIX,SOCK_STREAM,0);
 if(iFD<0)throw Error::Sock(METH_NAME,"cannot open socket");

 memset(reinterpret_cast<char*>(&saStand),0,sizeof(sockaddr_un));
 saStand.sun_family=AF_UNIX;
 sFile.copy(saStand.sun_path,sizeof(saStand.sun_path));
}

void Socket_basic::Close(){if(iFD>=0 && bMy){close(iFD);iFD=-1;}}

void Socket_basic::Listen(const std::string& sFile){
 sockaddr_un saStand;
 PrepareOpen(sFile,saStand);
 
 if(bind(iFD,reinterpret_cast<sockaddr*>(&saStand),sizeof(sockaddr_un))<0){
  Close();
  throw Error::Sock(METH_NAME,"cannot bind socket");
 }

 if(listen(iFD,1)<0){
  Close();
  throw Error::Sock(METH_NAME,"cannot listen");
 }
}

void Socket_basic::Connect(const std::string& sFile){
 sockaddr_un saStand;
 PrepareOpen(sFile,saStand);

 if(connect(iFD,reinterpret_cast<sockaddr*>(&saStand),sizeof(sockaddr_un))<0){
  Close();
  throw Error::Sock(METH_NAME,"cannot connect");
 }
 if(fcntl(iFD,F_SETFL,O_NONBLOCK)){
  Close();
  throw Error::Runtime(METH_NAME,"failed to open nonblocking");
 }
}

void Socket_basic::Accept(int iS){
 Close();bMy=true;

 sockaddr addr;
 socklen_t len=sizeof(struct sockaddr);
 iFD=accept(iS,&addr,&len);
 if(iFD==-1)throw Error::Runtime(METH_NAME,"error on accepting connection");
 if(fcntl(iFD,F_SETFL,O_NONBLOCK)){
  Close();
  throw Error::Runtime(METH_NAME,"failed to open nonblocking");
 }
}

void PollClass::AddSocket(int fd,short events){
 ++iF;
 if(iF>iS){
  pollfd* op=pPoll;
  pPoll=new pollfd[iF];
  std::copy(op,op+iS,pPoll);
  delete[] op;
  iS=iF;
 }
 pPoll[iF-1].fd=fd;
 pPoll[iF-1].events=events;
 pPoll[iF-1].revents=0;
}

void Input::Read(){
 while(Free()){
  size_t iS=((iLst<i1st)?Free():(iSize-iLst));
  if((iLst+iS)%iSize==i1st)--iS;

  int i=read(sbSo,cpB+iLst,iS);
  if(i<0)return;

  cpB[iLst+i]='\0';
  for(char* cp=strchr(cpB+iLst,cSep);cp;cp=strchr(cp+1,cSep))
   qNL.push_back(cp-cpB);

  iLst=(iLst+i)%iSize; 
  if(i!=int(iS))return;
 }
}

std::pair<std::string,bool> Input::GetLine(){
 if(qNL.empty()){
  if(!Free()){i1st=0;iLst=0;b2Long=true;}//reset und put 2long flag
  return std::make_pair(std::string(""),false);
 }
 std::string s(cpB+i1st,((qNL.front()<i1st)?iSize:qNL.front())-i1st);
 if(qNL.front()<i1st)s.append(cpB,qNL.front());
 i1st=(qNL.front()+1)%iSize;
 qNL.pop_front();

 if(b2Long){ //dont use the rest of a 2 long command
  b2Long=false;
  throw Error::Sock(METH_NAME,"line too long");
 }
 return std::make_pair(s,true);
}

void Output::Write(){
 int i=1;
 while(!slOut.empty() && i>0 && sbSo.IsOpen()){
  if(!slOut.front().empty()){
   size_t sL=slOut.front().length();
   if((i=write(sbSo,slOut.front().c_str(),sL))<0)return;
   if(i>0)slOut.front().erase(0,i);
   if(i!=int(sL))return;
  }

  if((i=write(sbSo,&cSep,1))<0)return;
  if(i==1)slOut.pop_front();
 }

}
