//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>
#include "vtql_base.h"
#include "error_base.h"
#include "vtql_utils.h"

extern "C" {
#undef export
#define W               40
#define H               25
#include "extern/export.h"
#include "extern/font.h"
#undef H
#undef W
#undef export
};
#include "../config.h"
const std::string& VTQL::sVersion=LIBVTQL_VERSION;

using namespace VTQL;


Connect::Connect(const std::string& s):Outp(sCon,1),Inp(sCon,1000){
 sCon.Connect(s);
 poll.AddSocket(sCon,0);
}
Connect::Connect(const sock::Socket_basic& s):
 sCon(int(s)),Outp(sCon,1),Inp(sCon,1000){
 poll.AddSocket(sCon,0);
}

Connect& Connect::operator <<(const std::string& s){Outp.Add(s);return *this;}

void Connect::Poll(bool bWait){
 bool bW=!Outp.Empty();
 bool bR=Inp.Free();

 if(bWait&&Inp.Lines())bWait=false;
 if(!bR && !bW && bWait)return;

 poll.Change(0,(bW?POLLOUT:0)|(bR?POLLIN:0));
 int i=poll.Run(bWait?-1:0);
 if(!i)return;
 if(i<0)throw Error::Runtime(METH_NAME,"error on poll");

 if(poll.Test(0,POLLHUP|POLLNVAL))
  throw Error::Runtime(METH_NAME,"socket closed");

 if(bW&&poll.Test(0,POLLOUT))Outp.Write();
 if(bR&&poll.Test(0,POLLIN))Inp.Read();
}

std::string Connect::FetchLine(){
 std::pair<std::string,bool> r=Inp.GetLine();
 if(!r.second)throw Error::Program(METH_NAME,"no line to fetch");
 return r.first;
}

bool CModul::Wait(bool b){
 if(!b)return qc.Process(false);
 while(!qc.Process(true));
 return true;
}

std::string VTQL::NextSep(char c,std::string& s){
 size_t i=0;
 if(s[0]=='\''){
  s.erase(0,1);
  while((i=s.find('\'',i))!=s.npos){
   s.erase(i,1);++i;
   if(i>=s.length()){i=s.npos;break;}
   if(s[i-1]!='\''){--i;break;}
  }
 }

 if(i!=s.npos)i=s.find(c,i);
 std::string sr;
 if(i!=s.npos){
  sr=s.substr(0,i);
  s.erase(0,i+1);
 }else{
  sr=s;
  s.clear();
 }
 return sr;
}

static void DecodePageInfo(std::string s,std::vector<long>& v){
 std::string s2=NextSep('.',s);
 v.push_back((s2=="unknown")?-1:strtol(s2.c_str(),NULL,10));
 s2=NextSep('.',s);
 v.push_back((s2=="unknown")?0:strtol(s2.c_str(),NULL,10));
 v.push_back(strtol(NextSep('.',s).c_str(),NULL,16));
 v.push_back(strtol(NextSep('.',s).c_str(),NULL,16));
 v.push_back(strtol(NextSep('.',s).c_str(),NULL,10)); //!!!
 v.push_back(strtol(NextSep('.',s).c_str(),NULL,16));

 while(!s.empty()){
  s2=NextSep(',',s);
  std::string s3=NextSep('.',s2);
  v.push_back(strtol(s3.c_str(),NULL,16));
  v.push_back(s2.empty()?ANY_SUB:strtol(s2.c_str(),NULL,16));
 }
}

bool AlevtPage::ReadLine(const std::string& s){
 if(iLC==VT_HEIGHT){
  std::vector<long> v;
  DecodePageInfo(s,v);
  pg.pgno=v[2];
  pg.subno=v[3];
  pg.flags=v[5];
  size_t i;
  for(i=6;i<v.size();i+=2){
   pg.link[(i-6)/2].pgno=v[i];
   pg.link[(i-6)/2].subno=v[i+1];
  }
  pg.flof=(i-6)/2;
  iLC=0;
  return true;
 }

 if(!iLC){pg.lines=0;pg.errors=0;}

 pg.lines|=1<<iLC;
 for(unsigned i=0;i<VT_WIDTH;++i){
  pg.data[iLC][i]=
   ((s[i*2  ]>='A')?(s[i*2  ]-'A'+10):(s[i*2  ]-'0'))*16+
   ((s[i*2+1]>='A')?(s[i*2+1]-'A'+10):(s[i*2+1]-'0'));
 }
 ++iLC;
 return false;
}

namespace VTQL{
struct eexport* epExpPng=NULL;
struct eexport* epExpAnsi=NULL;
}

size_t AlevtPage::GetPng(char* cp,int iC,bool bReveal){
 char cpp[50];
 if(!epExpPng){
  strcpy(cpp,"png");
  epExpPng=export_open(cpp);
 }

 sprintf(cpp,"%i",(iC<0)?0:((iC>9)?9:iC));
 epExpPng->mod->option(epExpPng,1,cpp);
 epExpPng->reveal=bReveal;
 int iL=export_caller(epExpPng,&pg,cp);
 if(iL<0)throw Error::Program(METH_NAME,"cannot create png");
 return iL;
}

std::string AlevtPage::GetAnsi(bool bReveal){
 char cpp[50];
 if(!epExpAnsi){strcpy(cpp,"ansi");epExpAnsi=export_open(cpp);}

 epExpAnsi->reveal=bReveal;
 char** cp=new char*[VT_HEIGHT];
 int iL=export_caller(epExpAnsi,&pg,reinterpret_cast<char*>(cp));
 if(iL<0)throw Error::Program(METH_NAME,"cannot create ansi");

 std::string s;
 for(unsigned ii=0;ii<VT_HEIGHT;++ii)s+=cp[ii];
 delete[] cp;
 return s;
}

PgInfo::PgInfo(const std::string& s){
 std::vector<long> v;
 DecodePageInfo(s,v);

 iSt=v[0];
 iFreq=v[1];
 iPage=v[2];
 iSPage=v[3];
 tTime=v[4];
 iFlags=v[5];
 for(size_t i=6;i<v.size();++i)
  if((i-6)%2)vFL.push_back(std::make_pair(v[i-1],v[i]));
}
