//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#include "vtql_parse.h"
#include "vtql_utils.h"
#include <cstring>
#include <fnmatch.h>

using std::string;
using namespace VTQL;

const int iRegexOpt=REG_EXTENDED|REG_ICASE|REG_NOSUB|REG_NEWLINE;
const int iGlobSearchOpt=FNM_CASEFOLD;

static int xstrtol(int iB,const char* cp){
 long lR=0;
 char* cpErr;
 lR=strtol(cp,&cpErr,iB);
 if(lR>=long(INT_MAX))lR=INT_MAX; //well. long==int. at the moment.
 if(lR<=long(INT_MIN))lR=INT_MIN;
 if(cpErr[0]!='\0')throw Error::Syntax(METH_NAME,"invalid number");
 return lR;
}

extern "C"{
#include "extern/frequencies.h"
}

static int xGetFreq(const string& s){
 if(s=="none")return -1;
 if(s=="unknown")return 0;
 size_t p=s.find("::");
 if(p==s.npos){
  int i=xstrtol(10,s.c_str());
  if(i<=0)throw Error::Syntax(METH_NAME,"invalid frequency");
  return i;
 }
 if(p==0 || (int(s.length())-int(p))<3)
  throw Error::Syntax(METH_NAME,"wrong frequency string format");
 string sTab=s.substr(0,p);
 string sSta=s.substr(p+2);

 p=sSta.find_last_of("~");
 int iOff=0;
 if(p!=sSta.npos){
  if((int(sSta.length())-int(p))<3 || p==0)
   throw Error::Syntax(METH_NAME,"wrong finetuning format");
  iOff=xstrtol(10,sSta.substr(p+2).c_str());
  if(iOff<0)throw Error::Syntax(METH_NAME,"invalid finetuneing");
  switch (sSta[p+1]){
   case 'm':case 'M':iOff=-iOff; //fall thru
   case 'p':case 'P':sSta.erase(p);break;
   default: iOff=0;break;
  }
 }

 int i;
 for(i=0;chanlists[i].name && sTab!=chanlists[i].name;++i);
 if(!chanlists[i].name)throw Error::Syntax(METH_NAME,"freqtab not found");

 int j;
 for(j=0;j<chanlists[i].count && sSta!=chanlists[i].list[j].name;++j);
 if(j==chanlists[i].count)throw Error::Syntax(METH_NAME,"channel not found");

 return chanlists[i].list[j].freq+(iOff*1000+8)/16;
}

static int xDecMode(const char* cp,const char* cppModeTable[]){
 for(size_t i=0;cppModeTable[i];++i)
  if(!strcasecmp(cp,cppModeTable[i]))return i;
 throw Error::Syntax(METH_NAME,"invalid mode");
}

Region_basic::Region_basic(const std::vector<size_t>& v)
:vBases(v),rvAll(v.size()),
 rSearch(NULL),cpFind(NULL),bGlobSearch(false),cpTT(new char[VT_WIDTH+1]){}

Region_basic::~Region_basic(){
 Remove_Regexp();
 delete[] cpFind;
 delete[] cpTT;
}

void Region_basic::ExtractRegex(string& s,const string& sModes){
 size_t l=s.find_first_of(sModes);
 if(l==s.npos)return;

 if(l+1<s.length()){ //not void
  cpFind=new char[s.substr(l+1).length()+1];
  strcpy(cpFind,s.substr(l+1).c_str());
  if(s[l]=='/'){
   rSearch=new regex_t;
   if(int i=regcomp(rSearch,cpFind,iRegexOpt)){
    char cp[1024];
    regerror(i,rSearch,cp,1024);
    delete rSearch;
    rSearch=NULL;
    throw Error::Syntax(METH_NAME,string("error in regexp:")+cp);
   }
  }else if(s[l]=='\\'){
   switch(fnmatch(cpFind,"dummy",iGlobSearchOpt)){
    case 0:case FNM_NOMATCH: break;
    default:throw Error::Syntax(METH_NAME,"error in glob match");
   }
   
   bGlobSearch=true;
  }
  if(!bGlobSearch)
   for(size_t i=0;cpFind[i];++i)cpFind[i]=VTQL::xtoupper(cpFind[i]);
 }
 s.erase(l); //delete search part
}

void Region_basic::NewString(string s,const string& sModes){
 rvAll=RangeVec(vBases.size());
 Remove_Regexp();
 delete[] cpFind;cpFind=NULL;
 bGlobSearch=false;

 ExtractRegex(s,sModes);

 for(size_t i=0;i<vBases.size();++i){
  for(int iUp=0;iUp>=0;){
   if(!iUp)rvAll[i].push_back(std::pair<int,int>(0,INT_MAX));

   size_t iPos=s.find_first_of(",-.");
   if(iPos && s.substr(0,iPos).length()){
    int l=(vBases[i]?
     xstrtol(vBases[i],s.substr(0,iPos).c_str()):xGetFreq(s.substr(0,iPos)));
    if(iUp)rvAll[i].back().second=l;
    else rvAll[i].back().first=rvAll[i].back().second=l;
   }else if(iUp)rvAll[i].back().second=INT_MAX;
   if(iPos!=s.npos){
    switch(s[iPos]){
     case ',':iUp=0;break;
     case '.':
      if(i+1==vBases.size())throw Error::Syntax(METH_NAME,"extra . at end");
      iUp=-1;
      break;
     case '-':
      if(iUp)throw Error::Syntax(METH_NAME,"doubled -");
      iUp=1;
      break;
    } 
    s.erase(0,iPos+1);
   }else {iUp=-1;s.erase();}
  }
 }
 if(s.length())throw Error::Syntax(METH_NAME,"invalid end of match string");
}

void Region_basic::Remove_Regexp(){
 if(rSearch)regfree(rSearch);
 delete rSearch;
 rSearch=NULL;
}

static const char cpFastSearch[]={
                         12,
              6,                     18,
        3,          9,         15,         21,
    1,    4,    7,   10,   13,   16,   19,   22, 24,
  0,  2,    5,    8,   11,   14,   17,   20,   23};
//0 1 2 3 4 5 6 7 8 9101112131415161718192021222324

bool Region_basic::MatchTxt(const PageData& pd)const{
 if(!HasTextSearch())return true;
 for(unsigned i=0;i<VT_HEIGHT;++i)if(IsIn(cpFastSearch[i]+1,4)){
  pd.GetLine(cpFastSearch[i],cpTT);
  if(rSearch){if(!regexec(rSearch,cpTT,0,NULL,0))return true;}
  else if(bGlobSearch){if(!fnmatch(cpFind,cpTT,iGlobSearchOpt))return true;}
  else if(VTQL::xstrcasestr(cpTT,cpFind))return true;
 }
 return false;
}

bool Region_basic::IsIn(int iWhat,int iWhere)const{
 for(SingleRange::const_iterator i=rvAll[iWhere].begin();
     i!=rvAll[iWhere].end();++i)
  if(i->first>=0 && i->first<=iWhat && i->second >=iWhat)return true;
 return false;
}

void Parse::NewString(const string& sCom){
 slWords.clear();
 string sIn=sCom;

 while(true){
  size_t i=sIn.find_first_not_of(' '); //begin of next word
  if(i==sIn.npos)return;
  sIn.erase(0,i);

  switch(sIn[0]){
   case '\'':
    for(i=1;;i+=2){
     if(i>=sIn.length() || (i=sIn.find_first_of('\'',i))==sIn.npos)
      throw Error::Syntax(METH_NAME,"no end of string");
     if((i+1>=sIn.length() || sIn[i+1]!='\''))break;
    }
    slWords.push_back(sIn.substr(0,i+1));
    sIn.erase(0,i+1);
    break;
   case '#': return;
   default:
    i=sIn.find_first_of(" #"); //end of word
    if(i==sIn.npos)i=sIn.length();
    slWords.push_back(sIn.substr(0,i));
    sIn.erase(0,i);
    break;
  }
 }
}

int Parse::GetCom()const{
 if(!slWords.size())return -1;

 for(int iC=0;cmAllCmds[iC][0].iArgs>=0;++iC){
  std::vector<string>::const_iterator iL=slWords.begin();
  bool bValid=true;

  for(int iW=0;cmAllCmds[iC][iW].iArgs>=0 && bValid;++iW){
   if(iL==slWords.end()){bValid=false;break;}
   if(strcasecmp(iL->c_str(),cmAllCmds[iC][iW].sWord))bValid=false;
   for(int i=0;i<cmAllCmds[iC][iW].iArgs && bValid;++i){
    ++iL;
    if(iL==slWords.end())bValid=false;
   }
   if(bValid)++iL;
  }
  if(bValid && iL==slWords.end())return iC;
 }
 throw Error::Syntax(METH_NAME,"invalid command");
}

int Parse::Int10(int iN,int iL,int iU)const{
 int i=xstrtol(10,slWords[iN].c_str());
 if(i<iL || i>iU)throw Error::Syntax(METH_NAME,"value not in range");
 return i;
}

string Parse::String(int iN)const{
 if(slWords[iN][0]!='\'' || slWords[iN].length() < 2)return slWords[iN];
 string s=slWords[iN].substr(1,slWords[iN].length()-2);
 size_t i=s.find_first_of('\'');
 while(i!=s.npos){
  s.erase(i,1);++i;
  if(i>=s.length())break;
  i=s.find_first_of('\'',i);
 }
 return s;
}

int Parse::Freq(int iN)const{return xGetFreq(slWords[iN]);}

template<>
VTQL::UserMode Parse::DecMode(int iN)const{
 return UserMode(
  xDecMode(slWords[iN].c_str(),cppUserModes)
 );
}

template<>
ShowMode Parse::DecMode(int iN)const{
 return ShowMode(
  xDecMode(slWords[iN].c_str(),cppShowModes)
 );
}
